#!/usr/bin/perl -w

###########################################################################
# Group  markers into different linkage groups using the ultra-fast version 
############################################i###############################
# Input : fasta   (de novo)
# Output: pair-end file
###########################################################################

use strict;
use warnings;
use Cwd 'abs_path';
use Time::localtime;
use DBM::Deep;
use Graph::Undirected;
use FindBin qw($Bin); 
use lib "$Bin/../lib";
use Foundation;

# Declare default values for configure

my $tf = new Foundation                ;
my $PRG = $tf->getProgramInfo('name')  ;
my $REV="1.0"                          ;
my @DEPENDS=("Foundation","ParseFasta");
my $VERSION = "Version 1.00"           ;

 
my $inputFile="NA";
my $outputDir="NA";
my $samplingNum;
my $groupThreshold;
my $clusterNum=100;



setDefaults()                          ;
setParametersFromCommandLine()         ;
my $threshold=$samplingNum*$groupThreshold;

printPrnLog("Loading the parameters succesfully\n");
system("mkdir -p $outputDir");
### A|B to 0|1
open (IN, "<$inputFile" )      ||  $tf->bail("Could't open $inputFile ($!)");
open (O, ">$outputDir/AB_01" )      ||  $tf->bail("Could't open $inputFile ($!)");
my %strToNum=(A=>"0",B=>"1",0=>"0",1=>"1");  
my %numToStr=(0=>"A",1=>"B",A=>"A",B=>"B");  
my %ID=();
my $cnt=0;
my $sampleSize=0;
my $g=Graph::Undirected->new;
my $chr="";

while(<IN>){
  chomp;
  if($_=~/-/){
      my @line=split(/\s+|\t/,$_);
      $cnt++;
      $ID{$cnt}=$_;
      print O "$line[0]";
      my @tmp=split(/-/,$_);
      $chr=$tmp[0];
      for(my $i=1;$i<@line;$i++){
      	  $sampleSize=@line-1;
          print O "\t$strToNum{$line[$i]}";
      }
      print O "\n";
  }
}
close (IN);
close (O);


### Get the cluster information for all markers 
system("less $outputDir/AB_01  | cut -f 1  | cut -f 2 -d \"-\" | sort -n  >  $outputDir/marker.sort");
open IN, "< $outputDir/marker.sort";
open O, "> $outputDir/mst.map";
my $LG=0;
my $last=0;
while(<IN>){
  chomp;
  if($LG==0){
    print O "lg$LG\n$chr"."-"."$_\n";
  }
  if($_-$last>20000){
    $LG++;
    print O "lg$LG\n$chr"."-"."$_\n";
  }
  else{
    print O "$chr-$_\n";
  }
  $last=$_;
}
close (IN);
close (O);



sub setParametersFromCommandLine{
    my $result = $tf->TIGR_GetOptions(
           "i=s"                     => \$inputFile,
           "o=s"                     => \$outputDir,
           "s=s"                     => \$samplingNum,
           "T=f"                     => \$groupThreshold,
           "c=n"                     => \$clusterNum
   );
   $tf  -> printUsageInfoAndExit() if (!$result);  
   $tf  -> printUsageInfoAndExit() if ($inputFile eq "NA" || $outputDir eq "NA");
}


sub setDefaults{
    my $HELPTEXT = qq~
Sort high-density markers in the ultra-fast mode.
Author : Jinzhuang Dou
Usage:   $PRG file -i  -o  -l -L [options]


Main parameters:

  -i    <s>     The input genotype file . 
  -o    <s>     The output file storing the generated results.
  -s    <s>     The maximal sampling times allowed for markers grouping and markers sorting.
  -T    <s>     Threshold over which pairs of contigs will be linked together under the specified sampling times. [0.6] 
  -c    <s>     A scalar denoting the number of clusters. [10]
  -h|help       print this help and exit;
  -V|version    print the version and exit;
  -depend       print the program and database dependency list;
  -debug        set the debug <level> (0, non-debug by default); 
        
OUTPUT:  
~;

my $MOREHELP = qq~
Return Codes:   0 - on success, 1 - on failure.
~;


    $tf->setHelpInfo($HELPTEXT.$MOREHELP);
    $tf->setUsageInfo($HELPTEXT);
    $tf->setVersionInfo($REV);
    $tf->addDependInfo(@DEPENDS);
}

sub printPrnLog
{
    my $msg = shift ;
    $tf -> logLocal($msg,1);
    print STDERR $msg; 

}
